@@ -78,6 +78,7 @@ static DICacheEntry *Dicache_entry_new(void)
    entry->Y = 0;
    entry->BitVec = NULL;
    entry->State = DIC_Empty;
+   entry->version = 0;
    entry->next = NULL;
 
    return entry;
@@ -226,7 +227,7 @@ void a_Dicache_invalidate_entry(const DilloUrl *Url)
 }
 
 
-// -------------------------------------------------------------------------
+/* ------------------------------------------------------------------------- */
 
 /*
  * This function is a cache client; (but feeds its clients from dicache)
@@ -264,7 +265,7 @@ void a_Dicache_callback(int Op, CacheClient_t *Client)
    }
 }
 
-// -------------------------------------------------------------------------
+/* ------------------------------------------------------------------------- */
 
 /*
  * Set image's width, height & type
@@ -304,13 +305,15 @@ void a_Dicache_set_parms(DilloUrl *url, gint version, DilloImage *Image,
  * Implement the set_cmap method for the Image
  */
 void a_Dicache_set_cmap(DilloUrl *url, gint version, DilloImage *Image,
-                        const guchar *cmap, gint num_colors, gint bg_index)
+                        const guchar *cmap, gint num_colors,
+                        gint num_colors_max, gint bg_index)
 {
    DICacheEntry *DicEntry = Dicache_get_entry_version(url, version);
 
    g_return_if_fail ( DicEntry != NULL );
 
-   DicEntry->cmap = g_realloc(DicEntry->cmap, 3 * num_colors);
+   g_free(DicEntry->cmap);
+   DicEntry->cmap = g_new0(guchar, 3 * num_colors_max);
    memcpy(DicEntry->cmap, cmap, 3 * num_colors);
    if (bg_index >= 0 && bg_index < num_colors) {
       DicEntry->cmap[bg_index * 3]     = (Image->bg_color >> 16) & 0xff;
@@ -364,7 +367,7 @@ void a_Dicache_close(DilloUrl *url, gint version, CacheClient_t *Client)
    a_Interface_close_client(Web->bw, Client->Key);
 }
 
-// -------------------------------------------------------------------------
+/* ------------------------------------------------------------------------- */
 
 static gboolean
  Dicache_remove_hash_entry(gpointer key, gpointer value, gpointer user_data)
