@@ -45,6 +45,7 @@
 #include "interface.h"
 #include "dw.h"
 #include "dw_widget.h"
+#include "cookies.h"
 
 
 /*
@@ -80,9 +81,11 @@ gint main(int argc, char *argv[])
    a_Dns_init();
    a_Http_init();
    a_Mime_init();
+   a_Cache_init();
    a_Dicache_init();
    a_Interface_init();
    a_Dw_init();
+   a_Cookies_init();
 
    /* a_Nav_init() has been moved into this call because it needs to be
     * initialized with the new browser_window structure */
@@ -124,6 +127,7 @@ gint main(int argc, char *argv[])
     * (This can be left to the OS, but we'll do it, with a view to test
     *  and fix our memory management)
     */
+   a_Cookies_freeall();
    a_Cache_freeall();
    a_Dicache_freeall();
    a_Http_freeall();
