@@ -476,6 +476,9 @@ static gint Dw_gtk_scrolled_frame_key_press (GtkWidget *widget,
          return gtk_container_focus (container, GTK_DIR_TAB_FORWARD);
 
    case GDK_Page_Up:
+   case GDK_b: case GDK_B:
+      if (event->state & (GDK_MOD1_MASK | GDK_MOD2_MASK))
+         return FALSE;
       if (event->state & GDK_CONTROL_MASK)
          Dw_gtk_scrolled_frame_move_by(frame,
                                       - frame->hadjustment->page_increment, 0);
@@ -485,6 +488,7 @@ static gint Dw_gtk_scrolled_frame_key_press (GtkWidget *widget,
       return TRUE;
 
    case GDK_Page_Down:
+   case GDK_space:
       if (event->state & GDK_CONTROL_MASK)
          Dw_gtk_scrolled_frame_move_by(frame,
                                       + frame->hadjustment->page_increment, 0);
