@@ -66,6 +66,7 @@ static gint Dw_page_motion_notify     (DwWidget *widget,
                                        gint32 y,
                                        GdkEventMotion *event);
 static gint Dw_page_leave_notify      (DwWidget *widget,
+                                       DwWidget *next_widget,
                                        GdkEventMotion *event);
 
 
@@ -97,7 +98,7 @@ static void Dw_page_rewrap            (DwPage *page);
  */
 #define Dw_page_line_total_x_offset(page, line) \
    ( Dw_page_line_x_offset (page, line) + \
-     Dw_style_box_offset_x (((DwWidget*)(page))->style) )
+     p_Dw_style_box_offset_x (((DwWidget*)(page))->style) )
 
 
 enum
@@ -214,7 +215,7 @@ static void Dw_page_class_init (DwPageClass *klass)
                       GTK_RUN_FIRST,
                       object_class->type,
                       GTK_SIGNAL_OFFSET (DwPageClass, link_entered),
-                      Dw_marshal_link_enter,
+                      p_Dw_marshal_link_enter,
                       GTK_TYPE_NONE,
                       3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_INT);
    page_signals[LINK_PRESSED] =
@@ -222,7 +223,7 @@ static void Dw_page_class_init (DwPageClass *klass)
                       GTK_RUN_FIRST,
                       object_class->type,
                       GTK_SIGNAL_OFFSET (DwPageClass, link_pressed),
-                      Dw_marshal_link_button,
+                      p_Dw_marshal_link_button,
                       GTK_TYPE_NONE,
                       4, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_INT,
                       GTK_TYPE_GDK_EVENT);
@@ -231,7 +232,7 @@ static void Dw_page_class_init (DwPageClass *klass)
                       GTK_RUN_FIRST,
                       object_class->type,
                       GTK_SIGNAL_OFFSET (DwPageClass, link_released),
-                      Dw_marshal_link_button,
+                      p_Dw_marshal_link_button,
                       GTK_TYPE_NONE,
                       4, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_INT,
                       GTK_TYPE_GDK_EVENT);
@@ -240,7 +241,7 @@ static void Dw_page_class_init (DwPageClass *klass)
                       GTK_RUN_FIRST,
                       object_class->type,
                       GTK_SIGNAL_OFFSET (DwPageClass, link_clicked),
-                      Dw_marshal_link_button,
+                      p_Dw_marshal_link_button,
                       GTK_TYPE_NONE,
                       4, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_INT,
                       GTK_TYPE_GDK_EVENT);
@@ -334,17 +335,17 @@ static void Dw_page_size_request (DwWidget *widget,
          at the end of a page are not visible. */
       height =
          last_line->top + last_line->ascent + last_line->descent +
-         Dw_style_box_rest_height (widget->style);
+         p_Dw_style_box_rest_height (widget->style);
       requisition->ascent = page->lines[0].top + page->lines[0].ascent;
       requisition->descent = height - requisition->ascent;
    } else {
       requisition->width = page->last_line_width;
-      requisition->ascent = Dw_style_box_offset_y (widget->style);
-      requisition->descent = Dw_style_box_rest_height (widget->style);
+      requisition->ascent = p_Dw_style_box_offset_y (widget->style);
+      requisition->descent = p_Dw_style_box_rest_height (widget->style);
    }
 
    requisition->width +=
-      page->inner_padding + Dw_style_box_diff_width (widget->style);
+      page->inner_padding + p_Dw_style_box_diff_width (widget->style);
    if (requisition->width < page->avail_width)
       requisition->width = page->avail_width;
 }
@@ -438,7 +439,7 @@ static void Dw_page_get_extremes (DwWidget *widget,
                word_extremes.min_width += page->line1_offset;
                word_extremes.max_width += page->line1_offset;
             }
-           
+
             if (nowrap)
                par_min += word_extremes.min_width + word->orig_space;
             else
@@ -484,9 +485,9 @@ static void Dw_page_get_extremes (DwWidget *widget,
    }
 
    extremes->min_width +=
-      page->inner_padding + Dw_style_box_diff_width (widget->style);
+      page->inner_padding + p_Dw_style_box_diff_width (widget->style);
    extremes->max_width +=
-      page->inner_padding + Dw_style_box_diff_width (widget->style);
+      page->inner_padding + p_Dw_style_box_diff_width (widget->style);
 }
 
 
@@ -568,7 +569,7 @@ static void Dw_page_set_width (DwWidget *widget,
       necessary. */
    if (page->avail_width != width || prefs.limit_text_width) {
       page->avail_width = width;
-      Dw_widget_queue_resize (widget, 0, FALSE);
+      p_Dw_widget_queue_resize (widget, 0, FALSE);
       page->must_queue_resize = FALSE;
    }
 }
@@ -586,7 +587,7 @@ static void Dw_page_set_ascent (DwWidget *widget,
 
    if (page->avail_ascent != ascent) {
       page->avail_ascent = ascent;
-      Dw_widget_queue_resize (widget, 0, FALSE);
+      p_Dw_widget_queue_resize (widget, 0, FALSE);
       page->must_queue_resize = FALSE;
    }
 }
@@ -604,7 +605,7 @@ static void Dw_page_set_descent (DwWidget *widget,
 
    if (page->avail_descent != descent) {
       page->avail_descent = descent;
-      Dw_widget_queue_resize (widget, 0, FALSE);
+      p_Dw_widget_queue_resize (widget, 0, FALSE);
       page->must_queue_resize = FALSE;
    }
 }
@@ -666,7 +667,7 @@ static void Dw_page_word_wrap (DwPage *page, gint word_ind)
    DwExtremes word_extremes;
 
    avail_width =
-      page->avail_width - Dw_style_box_diff_width(widget->style)
+      page->avail_width - p_Dw_style_box_diff_width(widget->style)
       - page->inner_padding;
    if (prefs.limit_text_width &&
        avail_width > widget->viewport->allocation.width - 10)
@@ -717,12 +718,12 @@ static void Dw_page_word_wrap (DwPage *page, gint word_ind)
          orig_space_sum = 0;
          for (i = last_line->first_word; i < last_line->last_word - 1; i++)
             orig_space_sum += page->words[i].orig_space;
-        
+
          orig_space_cum = 0;
          last_eff_space_diff_cum = 0;
          for (i = last_line->first_word; i < last_line->last_word - 1; i++) {
             orig_space_cum += page->words[i].orig_space;
-            
+
             if (orig_space_cum == 0)
                eff_space_diff_cum = last_eff_space_diff_cum;
             else
@@ -734,13 +735,12 @@ static void Dw_page_word_wrap (DwPage *page, gint word_ind)
             last_eff_space_diff_cum = eff_space_diff_cum;
          }
       }
-   }         
+   }
 
    if (new_line) {
       /* Add a new line. */
       page->num_lines++;
-      a_List_add(page->lines, page->num_lines, sizeof(DwPageLine),
-                 page->num_lines_max);
+      a_List_add(page->lines, page->num_lines, page->num_lines_max);
 
       last_line = &page->lines[page->num_lines - 1];
 
@@ -761,7 +761,7 @@ static void Dw_page_word_wrap (DwPage *page, gint word_ind)
          last_line->par_max = plast_line->par_max;
       } else {
          /* first line: initialize values */
-         last_line->top = Dw_style_box_offset_y (DW_WIDGET(page)->style);
+         last_line->top = p_Dw_style_box_offset_y (DW_WIDGET(page)->style);
          last_line->max_line_width = page->line1_offset;
          last_line->max_word_min = 0;
          last_line->max_par_max = 0;
@@ -831,15 +831,15 @@ static void Dw_page_word_wrap (DwPage *page, gint word_ind)
       case DW_STYLE_TEXT_ALIGN_STRING:   /* handled elsewhere (in future) */
          left_offset = 0;
          break;
-         
+
       case DW_STYLE_TEXT_ALIGN_RIGHT:
          left_offset = avail_width - page->last_line_width;
          break;
-         
+
       case DW_STYLE_TEXT_ALIGN_CENTER:
          left_offset = (avail_width - page->last_line_width) / 2;
          break;
-         
+
       default:
          /* compiler happiness */
          left_offset = 0;
@@ -873,9 +873,9 @@ static void Dw_page_calc_widget_size (DwPage *page,
    gint32 avail_width, avail_ascent, avail_descent;
 
    avail_width =
-      page->avail_width - Dw_style_box_diff_width(DW_WIDGET(page)->style);
+      page->avail_width - p_Dw_style_box_diff_width(DW_WIDGET(page)->style);
    avail_ascent =
-      page->avail_ascent - Dw_style_box_diff_height(DW_WIDGET(page)->style);
+      page->avail_ascent - p_Dw_style_box_diff_height(DW_WIDGET(page)->style);
    avail_descent = page->avail_descent;
 
    if (DW_WIDGET_USES_HINTS (widget)) {
@@ -893,7 +893,7 @@ static void Dw_page_calc_widget_size (DwPage *page,
       else if (DW_STYLE_IS_LENGTH (widget->style->width))
          size->width =
             DW_STYLE_GET_LENGTH (widget->style->width, widget->style->font)
-            + Dw_style_box_diff_width (widget->style);
+            + p_Dw_style_box_diff_width (widget->style);
       else
          size->width =
             DW_STYLE_GET_PERCENTAGE (widget->style->width) *  avail_width;
@@ -904,7 +904,7 @@ static void Dw_page_calc_widget_size (DwPage *page,
       } else if (DW_STYLE_IS_LENGTH (widget->style->width)) {
          size->ascent =
             DW_STYLE_GET_LENGTH (widget->style->height, widget->style->font)
-            + Dw_style_box_diff_height (widget->style);
+            + p_Dw_style_box_diff_height (widget->style);
          size->descent = 0;
       } else {
          size->ascent =
@@ -951,8 +951,8 @@ static void Dw_page_rewrap(DwPage *page)
          word->content.widget->parent_ref = page->num_lines - 1;
 
       if ( word->content_type == DW_PAGE_CONTENT_ANCHOR ) {
-         Dw_widget_set_anchor(widget, word->content.anchor,
-                              page->lines[page->num_lines - 1].top);
+         p_Dw_widget_set_anchor(widget, word->content.anchor,
+                                page->lines[page->num_lines - 1].top);
       }
    }
 }
@@ -988,13 +988,13 @@ static void Dw_page_draw_line (DwPage *page,
    widget = DW_WIDGET (page);
    window = DW_WIDGET_WINDOW (widget);
    x_cursor =
-      Dw_widget_x_world_to_viewport (widget,
-                                     widget->allocation.x +
-                                     Dw_page_line_total_x_offset(page, line));
+      p_Dw_widget_x_world_to_viewport (widget,
+                                       widget->allocation.x +
+                                       Dw_page_line_total_x_offset(page,line));
    y_cursor =
-      Dw_widget_y_world_to_viewport (widget,
-                                     widget->allocation.y
-                                     + line->top + line->ascent);
+      p_Dw_widget_y_world_to_viewport (widget,
+                                       widget->allocation.y
+                                       + line->top + line->ascent);
 
    for (word_index = line->first_word; word_index < line->last_word;
         word_index++) {
@@ -1016,32 +1016,30 @@ static void Dw_page_draw_line (DwPage *page,
                          word->content.text);
 
          /* underline */
-         if (word->style->uline >= 0)
+         if (word->style->uline)
             gdk_draw_line(window,
                           word->style->color->gc,
                           x_cursor,
                           y_cursor + 1 + diff,
                           x_cursor + word->size.width - 1,
                           y_cursor + 1 + diff);
-         if (word_index + 1 < line->last_word &&
-             word->space_style->uline >= 0)
+         if (word_index + 1 < line->last_word && word->space_style->uline)
             gdk_draw_line(window,
                           word->style->color->gc,
                           x_cursor + word->size.width,
                           y_cursor + 1 + diff,
                           x_cursor + word->size.width + word->eff_space - 1,
                           y_cursor + 1 + diff);
-         
+
          /* strike-through */
-         if (word->style->strike >= 0)
+         if (word->style->strike)
             gdk_draw_line(window,
                           word->style->color->gc,
                           x_cursor,
                           y_cursor - word->size.ascent / 2 + diff,
                           x_cursor + word->size.width - 1,
                           y_cursor - word->size.ascent / 2 + diff);
-         if (word_index + 1 < line->last_word &&
-             word->space_style->strike >= 0)
+         if (word_index + 1 < line->last_word && word->space_style->strike)
             gdk_draw_line(window,
                           word->style->color->gc,
                           x_cursor + word->size.width,
@@ -1053,7 +1051,7 @@ static void Dw_page_draw_line (DwPage *page,
 
       case DW_PAGE_CONTENT_WIDGET:
          child = word->content.widget;
-         if (Dw_widget_intersect (child, area, &child_area))
+         if (p_Dw_widget_intersect (child, area, &child_area))
             a_Dw_widget_draw (child, &child_area, event);
 
          break;
@@ -1078,9 +1076,8 @@ static void Dw_page_draw_line (DwPage *page,
          break;
 
       default:
-         //g_print ("                          BUG!!! at (%d, %d).\n",
-         //         x_cursor, y_cursor + diff);
-
+         g_warning ("BUG!!! at (%d, %d).", x_cursor, y_cursor + diff);
+         break;
       }
       x_cursor += word->size.width + word->eff_space;
    }
@@ -1129,7 +1126,7 @@ static void Dw_page_draw (DwWidget *widget,
    gint line_index;
    DwPageLine *line;
 
-   Dw_widget_draw_widget_box (widget, area);
+   p_Dw_widget_draw_widget_box (widget, area);
 
    page = DW_PAGE (widget);
    line_index = Dw_page_find_line_index(page, area->y);
@@ -1262,6 +1259,7 @@ static gint Dw_page_motion_notify (DwWidget *widget,
  * Standard Dw function.
  */
 static gint Dw_page_leave_notify (DwWidget *widget,
+                                  DwWidget *next5_widget,
                                   GdkEventMotion *event)
 {
    DwPage *page = DW_PAGE (widget);
@@ -1288,8 +1286,7 @@ static DwPageWord *Dw_page_add_word(DwPage *page,
    DwPageWord *word;
 
    page->num_words++;
-   a_List_add(page->words, page->num_words, sizeof(*page->words),
-              page->num_words_max);
+   a_List_add(page->words, page->num_words, page->num_words_max);
 
    word = &page->words[page->num_words - 1];
    word->size.width = width;
@@ -1344,6 +1341,7 @@ void a_Dw_page_add_widget (DwPage *page,
    DwPageWord *word;
    DwRequisition size;
 
+   p_Dw_widget_set_parent (widget, DW_WIDGET (page));
    a_Dw_widget_set_style (widget, style);
 
    Dw_page_calc_widget_size (page, widget, &size);
@@ -1352,7 +1350,6 @@ void a_Dw_page_add_widget (DwPage *page,
    word->content_type = DW_PAGE_CONTENT_WIDGET;
    word->content.widget = widget;
 
-   Dw_widget_set_parent (widget, DW_WIDGET (page));
    Dw_page_word_wrap (page, page->num_words - 1);
    word->content.widget->parent_ref = page->num_lines - 1;
 }
@@ -1371,8 +1368,8 @@ void a_Dw_page_add_anchor(DwPage *page, const char *name, DwStyle *style)
    word->content.anchor = g_strdup(name);
    Dw_page_word_wrap (page, page->num_words - 1);
 
-   Dw_widget_set_anchor(DW_WIDGET(page), word->content.anchor,
-                        page->lines[page->num_lines - 1].top);
+   p_Dw_widget_set_anchor(DW_WIDGET(page), word->content.anchor,
+                          page->lines[page->num_lines - 1].top);
 }
 
 
@@ -1401,9 +1398,9 @@ void a_Dw_page_add_space(DwPage *page, DwStyle *style)
 
 
 /*
- * Cause a break
+ * Cause a paragraph break
  */
-void a_Dw_page_add_break(DwPage *page, gint space, DwStyle *style)
+void a_Dw_page_add_parbreak(DwPage *page, gint space, DwStyle *style)
 {
    DwPageWord *word, *word2;
    DwWidget *widget;
@@ -1442,7 +1439,7 @@ void a_Dw_page_add_break(DwPage *page, gint space, DwStyle *style)
                 word2->content_type == DW_PAGE_CONTENT_BREAK) {
                if(word2->content.break_space < space) {
                   word2->content.break_space = space;
-                  Dw_widget_queue_resize (DW_WIDGET(page2), lineno, FALSE);
+                  p_Dw_widget_queue_resize (DW_WIDGET(page2), lineno, FALSE);
                   page2->must_queue_resize = FALSE;
                }
             }
@@ -1467,6 +1464,22 @@ void a_Dw_page_add_break(DwPage *page, gint space, DwStyle *style)
    Dw_page_word_wrap (page, page->num_words - 1);
 }
 
+/*
+ * Cause a line break.
+ */
+void a_Dw_page_add_linebreak (DwPage *page, DwStyle *style)
+{
+   DwPageWord *word;
+
+   word = Dw_page_add_word(page, 0, style->font->font->ascent,
+                           style->font->font->descent, style);
+   word->content_type = DW_PAGE_CONTENT_BREAK;
+   word->content.break_space = 0;
+   word->style = style;
+   //a_Dw_style_ref (style);
+   Dw_page_word_wrap (page, page->num_words - 1);
+}
+
 /*
  * This function "hands" the last break of a page "over" to a parent
  * page. This is used for "collapsing spaces".
@@ -1482,7 +1495,7 @@ void a_Dw_page_hand_over_break (DwPage *page, DwStyle *style)
    last_line = &page->lines[page->num_lines - 1];
    if (last_line->break_space != 0 &&
        (parent = DW_WIDGET(page)->parent) && DW_IS_PAGE (parent))
-      a_Dw_page_add_break (DW_PAGE (parent), last_line->break_space, style);
+      a_Dw_page_add_parbreak (DW_PAGE (parent), last_line->break_space, style);
 }
 
 /*
@@ -1491,7 +1504,7 @@ void a_Dw_page_hand_over_break (DwPage *page, DwStyle *style)
 void a_Dw_page_flush (DwPage *page)
 {
    if (page->must_queue_resize) {
-      Dw_widget_queue_resize (DW_WIDGET(page), -1, TRUE);
+      p_Dw_widget_queue_resize (DW_WIDGET(page), -1, TRUE);
       page->must_queue_resize = FALSE;
    }
 }
