@@ -158,8 +158,8 @@ void a_Dw_style_init_values (DwStyle *style_attrs,
                              GdkWindow *window)
 {
    style_attrs->link = -1;
-   style_attrs->uline = -1;
-   style_attrs->strike = -1;
+   style_attrs->uline = FALSE;
+   style_attrs->strike = FALSE;
    style_attrs->SubSup = -1;
    style_attrs->nowrap = FALSE;
    style_attrs->text_align = DW_STYLE_TEXT_ALIGN_LEFT;
@@ -334,6 +334,8 @@ static DwStyleFont* Dw_style_font_new_internal (DwStyleFont *font_attrs,
 {
    DwStyleFont *font;
 
+   g_return_val_if_fail (font_attrs->name != NULL, NULL);
+
    if ((font = g_hash_table_lookup (fonts_table, font_attrs))) {
       return font;
    } else {
@@ -405,7 +407,7 @@ DwStyleFont* a_Dw_style_font_new_from_list (DwStyleFont *font_attrs)
    g_free (list);
 
    if (font == NULL) {
-      font_attrs2.name = "helvetica";
+      font_attrs2.name = prefs.vw_fontname;
       font = Dw_style_font_new_internal (&font_attrs2, TRUE);
    }
 
@@ -780,15 +782,15 @@ static void Dw_style_draw_polygon (GdkDrawable *drawable,
 #define LIMIT(v)    if ((v) < -30000) v = -30000; \
                     if ((v) > 30000) v = 30000
 
-void Dw_style_draw_border (GdkWindow *window,
-                           GdkRectangle *area,
-                           gint32 vx,
-                           gint32 vy,
-                           gint32 x,
-                           gint32 y,
-                           gint32 width,
-                           gint32 height,
-                           DwStyle *style)
+void p_Dw_style_draw_border (GdkWindow *window,
+                             GdkRectangle *area,
+                             gint32 vx,
+                             gint32 vy,
+                             gint32 x,
+                             gint32 y,
+                             gint32 width,
+                             gint32 height,
+                             DwStyle *style)
 {
    /* todo: a lot! */
    GdkGC *top_gc, *right_gc, *bottom_gc, *left_gc;
@@ -860,15 +862,15 @@ void Dw_style_draw_border (GdkWindow *window,
  * according to style. Used by Dw_widget_draw_box and
  * Dw_widget_draw_widget_box.
  */
-void Dw_style_draw_background (GdkWindow *window,
-                               GdkRectangle *area,
-                               gint32 vx,
-                               gint32 vy,
-                               gint32 x,
-                               gint32 y,
-                               gint32 width,
-                               gint32 height,
-                               DwStyle *style)
+void p_Dw_style_draw_background (GdkWindow *window,
+                                 GdkRectangle *area,
+                                 gint32 vx,
+                                 gint32 vy,
+                                 gint32 x,
+                                 gint32 y,
+                                 gint32 width,
+                                 gint32 height,
+                                 DwStyle *style)
 {
    DwRectangle dw_area, bg_area, intersection;
 
@@ -887,7 +889,7 @@ void Dw_style_draw_background (GdkWindow *window,
          height - style->margin.top - style->border_width.top -
          style->margin.bottom - style->border_width.bottom;
 
-      if (Dw_rectangle_intersect (&dw_area, &bg_area, &intersection))
+      if (p_Dw_rectangle_intersect (&dw_area, &bg_area, &intersection))
          gdk_draw_rectangle (window, style->background_color->gc, TRUE,
                              intersection.x - vx, intersection.y - vy,
                              intersection.width, intersection.height);
