@@ -286,7 +286,7 @@ static void Dw_table_size_request (DwWidget *widget,
                  avail_width);
 
       avail_width -= (2 * widget->style->border_spacing +
-                      Dw_style_box_diff_width (widget->style));
+                      p_Dw_style_box_diff_width (widget->style));
       Dw_table_sub_calc_col_widths
          (table->sub, avail_width,
           avail_width - (table->num_cols - 1) * widget->style->border_spacing);
@@ -295,7 +295,7 @@ static void Dw_table_size_request (DwWidget *widget,
    }
 
    /* Calculate the total width. */
-   requisition->width = Dw_style_box_diff_width (widget->style) +
+   requisition->width = p_Dw_style_box_diff_width (widget->style) +
       (table->num_cols + 1) * widget->style->border_spacing;
    for (col = 0; col < table->num_cols; col++)
       requisition->width += table->col_width[col];
@@ -324,34 +324,30 @@ static void Dw_table_size_request (DwWidget *widget,
             child_height =
                child_requisition.ascent + child_requisition.descent;
 
-            /*
-             * Adjust the row offsets. This is *always* done from the
-             * current position to the bottom, so this is perhaps
-             * worth to be optimized (?).
-             */
-            for (row2 = row + table->children[n]->data.cell.rowspan;
-                 row2 <= table->num_rows ;
-                 row2++)
-               if ( table->cum_height[row2] <
-                    table->cum_height[row] + child_height
-                    + widget->style->border_spacing ) {
-                  table->cum_height[row2] =
-                     table->cum_height[row] + child_height
-                     + widget->style->border_spacing;
-                  DEBUG_MSG (1, "Row %d starts at %d.\n",
-                             row2,
-                             table->cum_height[row +
-                                           table->children[n]
-                                           ->data.cell.rowspan]);
-               }
+            row2 = row + table->children[n]->data.cell.rowspan;
 
-            /* todo: column widths should be adjusted a bit. */
+            /* row2 is the row directly forced downward by the child */
+            if (table->cum_height[row2] <
+                table->cum_height[row] + child_height
+                + widget->style->border_spacing ) {
+               table->cum_height[row2] =
+                  table->cum_height[row] + child_height
+                  + widget->style->border_spacing;
+               DEBUG_MSG (1, "Row %d starts at %d.\n",
+                          row2, table->cum_height[row + table->children[n]
+                          ->data.cell.rowspan]);
+               /* todo: column widths should be adjusted a bit. */
+            }
          }
       }
-   }
+      /* if we've hit a row that contains no defined children, we need
+         to force cumulative heights to trickle down to lower rows */
 
+      if (table->cum_height[row + 1] < table->cum_height[row])
+         table->cum_height[row + 1] = table->cum_height[row];
+   }
    requisition->ascent = table->cum_height[table->num_rows] +
-                         Dw_style_box_diff_height (widget->style);
+                         p_Dw_style_box_diff_height (widget->style);
    requisition->descent = 0;
 
    DEBUG_MSG (2, "Dw_table_size_request: %d x %d x %d\n",
@@ -378,16 +374,16 @@ static void Dw_table_get_extremes (DwWidget *widget,
 
       Dw_table_sub_get_extremes (table->sub);
       diff = 2 * widget->style->border_spacing +
-         Dw_style_box_diff_width (widget->style);
+         p_Dw_style_box_diff_width (widget->style);
       extremes->min_width = table->sub->total_extremes.min_width + diff;
       extremes->max_width = table->sub->total_extremes.max_width + diff;
 
       DEBUG_MSG (DEBUG_EXTR_LEVEL + 2, "<-- END\n");
    } else {
       extremes->min_width = widget->style->border_spacing
-         + Dw_style_box_diff_width (widget->style);
+         + p_Dw_style_box_diff_width (widget->style);
       extremes->max_width = widget->style->border_spacing
-         + Dw_style_box_diff_width (widget->style);
+         + p_Dw_style_box_diff_width (widget->style);
    }
 }
 
@@ -412,11 +408,11 @@ static void Dw_table_size_allocate (DwWidget *widget,
    DEBUG_MSG (2, "Dw_table_size_allocate: %d x %d x %d\n",
               allocation->width, allocation->ascent, allocation->descent);
 
-   vdiff = allocation->y + Dw_style_box_offset_y (widget->style);
+   vdiff = allocation->y + p_Dw_style_box_offset_y (widget->style);
 
    for (row = 0; row < table->num_rows; row++) {
       col_offset = allocation->x + widget->style->border_spacing
-         + Dw_style_box_offset_x (widget->style);
+         + p_Dw_style_box_offset_x (widget->style);
 
       for (col = 0; col < table->num_cols; col++) {
          n = col + row * table->num_cols;
@@ -458,7 +454,7 @@ static void Dw_table_set_width (DwWidget *widget,
       necessary. */
    if (table->avail_width != width || prefs.limit_text_width) {
       table->avail_width = width;
-      Dw_widget_queue_resize (widget, 0, FALSE);
+      p_Dw_widget_queue_resize (widget, 0, FALSE);
    }
 }
 
@@ -474,7 +470,7 @@ static void Dw_table_set_ascent (DwWidget *widget,
    table = DW_TABLE (widget);
    if (table->avail_ascent != ascent) {
       table->avail_ascent = ascent;
-      Dw_widget_queue_resize (widget, 0, FALSE);
+      p_Dw_widget_queue_resize (widget, 0, FALSE);
    }
 }
 
@@ -490,7 +486,7 @@ static void Dw_table_set_descent (DwWidget *widget,
    table = DW_TABLE (widget);
    if (table->avail_descent != descent) {
       table->avail_descent = descent;
-      Dw_widget_queue_resize (widget, 0, FALSE);
+      p_Dw_widget_queue_resize (widget, 0, FALSE);
    }
 }
 
@@ -508,23 +504,23 @@ static void Dw_table_draw (DwWidget *widget,
    gint i;
    gint32 offx, offy, width;
 
-   Dw_widget_draw_widget_box (widget, area);
+   p_Dw_widget_draw_widget_box (widget, area);
 
-   offx = Dw_style_box_offset_x (widget->style);
-   offy = Dw_style_box_offset_y (widget->style);
-   width = widget->allocation.width - Dw_style_box_diff_width(widget->style);
+   offx = p_Dw_style_box_offset_x (widget->style);
+   offy = p_Dw_style_box_offset_y (widget->style);
+   width = widget->allocation.width - p_Dw_style_box_diff_width(widget->style);
    for (i = 0; i < table->num_rows; i++)
       if (table->row_style[i])
-         Dw_widget_draw_box (widget, table->row_style[i], area,
-                             offx, offy + table->cum_height[i],
-                             width,
-                             table->cum_height[i + 1] - table->cum_height[i]
-                             - widget->style->border_spacing);
+         p_Dw_widget_draw_box (widget, table->row_style[i], area,
+                               offx, offy + table->cum_height[i],
+                               width,
+                               table->cum_height[i + 1] - table->cum_height[i]
+                               - widget->style->border_spacing);
 
    for (i = 0; i < table->num_children; i++)
       if (CHILD_DEFINED (table, i)) {
          child = table->children[i]->data.cell.widget;
-         if (Dw_widget_intersect (child, area, &child_area)) {
+         if (p_Dw_widget_intersect (child, area, &child_area)) {
 #ifdef BORDERS
             gdk_draw_rectangle
                (DW_WIDGET_WINDOW (child),
@@ -658,15 +654,13 @@ static void Dw_table_realloc_children (DwTable *table)
    gint old_num_children, i;
 
    table->num_rows++;
-   a_List_resize (table->cum_height, table->num_rows, sizeof(gint32),
-                  table->cum_height_max);
+   a_List_resize (table->cum_height, table->num_rows, table->cum_height_max);
    table->num_rows--;
-   a_List_resize (table->row_style, table->num_rows, sizeof(gint32),
-                  table->row_style_max);
+   a_List_resize (table->row_style, table->num_rows, table->row_style_max);
 
    old_num_children = table->num_children;
    table->num_children = table->num_cols * table->num_rows;
-   a_List_resize (table->children, table->num_children, sizeof(DwTableChild*),
+   a_List_resize (table->children, table->num_children,
                   table->num_children_max);
 
    for (i = old_num_children; i < table->num_children; i++)
@@ -713,12 +707,12 @@ void a_Dw_table_add_cell (DwTable *table,
    if (table->cur_col + colspan > table->num_cols) {
       old_num_cols = table->num_cols;
       table->num_cols = table->cur_col + colspan;
-      a_List_resize (table->col_width, table->num_cols, sizeof(gint32),
+      a_List_resize (table->col_width, table->num_cols,
                      table->num_col_width_max);
 
       table->num_children = table->num_cols * table->num_rows;
       a_List_resize (table->children, table->num_children,
-                     sizeof(DwTableChild*), table->num_children_max);
+                     table->num_children_max);
 
       for (row = table->num_rows - 1; row >= 0; row--) {
          for (col = old_num_cols - 1; col >= 0; col--) {
@@ -751,8 +745,8 @@ void a_Dw_table_add_cell (DwTable *table,
    table->children[table->cur_row * table->num_cols + table->cur_col] = child;
    table->cur_col += colspan;
 
-   Dw_widget_set_parent (widget, DW_WIDGET (table));
-   Dw_widget_queue_resize (widget, 0, TRUE);
+   p_Dw_widget_set_parent (widget, DW_WIDGET (table));
+   p_Dw_widget_queue_resize (widget, 0, TRUE);
 
    /* The cell structure has changed, so the subtables have to be rebuilt. */
    if (table->sub) {
