@@ -51,8 +51,10 @@ static gint Dw_widget_real_motion_notify     (DwWidget *widget,
                                               gint32 y,
                                               GdkEventMotion *event);
 static gint Dw_widget_real_enter_notify      (DwWidget *widget,
+                                              DwWidget *last_widget,
                                               GdkEventMotion *event);
 static gint Dw_widget_real_leave_notify      (DwWidget *widget,
+                                              DwWidget *next_widget,
                                               GdkEventMotion *event);
 
 static void Dw_widget_update_cursor          (DwWidget *widget);
@@ -452,6 +454,7 @@ static gint Dw_widget_real_motion_notify (DwWidget *widget,
  * Standard Dw function
  */
 static gint Dw_widget_real_enter_notify (DwWidget *widget,
+                                         DwWidget *last_widget,
                                          GdkEventMotion *event)
 {
    /* not handled */
@@ -463,6 +466,7 @@ static gint Dw_widget_real_enter_notify (DwWidget *widget,
  * Standard Dw function
  */
 static gint Dw_widget_real_leave_notify (DwWidget *widget,
+                                         DwWidget *next_widget,
                                          GdkEventMotion *event)
 {
    /* not handled */
@@ -615,7 +619,8 @@ gint Dw_widget_mouse_event (DwWidget *widget,
    DwWidgetClass *klass;
    GtkDwViewport *viewport = GTK_DW_VIEWPORT (viewwidget);
    GdkEventType event_type;
-   DwWidget *ancestor, *w1, *w2;
+   DwWidget *ancestor, *w1, *w2, **track;
+   gint track_len, i;
 
    /* simulate crossing events */
    /* todo: resizing/moving widgets */
@@ -633,23 +638,45 @@ gint Dw_widget_mouse_event (DwWidget *widget,
                   ancestor = w1;
       }
 
-      /* leave_notify_event is called for all widgets down to the next
-         common ancestor. */
-      for (w1 = viewport->last_entered; w1 != ancestor; w1 = w1->parent) {
-         klass = DW_WIDGET_CLASS (GTK_OBJECT(w1)->klass);
-         klass->leave_notify_event (w1, (GdkEventMotion*) event);
+      /* Construct the track:
+       * viewport->last_entered => anchestor => (current) widget */
+      track_len = 0;
+      if (viewport->last_entered)
+         for(w1 = viewport->last_entered; w1 != ancestor; w1 = w1->parent)
+            track_len++;
+      if (ancestor)
+         track_len++; /* for the ancestor */
+      if (widget)
+         for(w1 = widget; w1 != ancestor; w1 = w1->parent)
+            track_len++;
+
+      track = g_new (DwWidget*, track_len);
+      i = 0;
+      if (viewport->last_entered)
+         for(w1 = viewport->last_entered; w1 != ancestor; w1 = w1->parent)
+            track[i++] = w1;
+      if (ancestor)
+         track[i++] = ancestor;
+      if (widget) {
+         i = track_len - 1;
+         for(w1 = widget; w1 != ancestor; w1 = w1->parent)
+            track[i--] = w1;
       }
 
-      viewport->last_entered = widget;
+      /* Send events to all events on the track */
+      for (i = 0; i < track_len; i++) {
+         klass = DW_WIDGET_CLASS (GTK_OBJECT(track[i])->klass);
+         if (i != 0)
+            klass->enter_notify_event (track[i], track[i - 1],
+                                       (GdkEventMotion*) event);
+         if (i != track_len - 1)
+            klass->leave_notify_event (track[i], track[i + 1],
+                                       (GdkEventMotion*) event);
+      }
 
-      /* todo: enter_notify_event is much simpler, but the mechanism used
-         for leave_notify_event is (currently) not needed in this case. */
-      if (widget) {
-         klass = DW_WIDGET_CLASS (GTK_OBJECT (widget)->klass);
-         (* (klass->enter_notify_event)) (widget, (GdkEventMotion*) event);
-         Dw_widget_update_cursor (widget);
-      } else
-         gdk_window_set_cursor (GTK_LAYOUT(viewport)->bin_window, NULL);
+      g_free (track);
+      viewport->last_entered = widget;
+      gdk_window_set_cursor (GTK_LAYOUT(viewport)->bin_window, NULL);
    }
 
    /* other events */
@@ -779,9 +806,9 @@ void a_Dw_widget_scroll_to (DwWidget *widget,
  * ...
  * The function has been copied from gdktrectangle.c
  */
-gint Dw_rectangle_intersect (DwRectangle *src1,
-                             DwRectangle *src2,
-                             DwRectangle *dest)
+gint p_Dw_rectangle_intersect (DwRectangle *src1,
+                               DwRectangle *src2,
+                               DwRectangle *dest)
 {
    DwRectangle *temp;
    gint src1_x2, src1_y2;
@@ -844,9 +871,9 @@ gint Dw_rectangle_intersect (DwRectangle *src1,
  * intersection (in widget coordinates!). Typically used by containers when
  * drawing their children. Returns whether intersection is not empty.
  */
-gint Dw_widget_intersect (DwWidget *widget,
-                          DwRectangle *area,
-                          DwRectangle *intersection)
+gint p_Dw_widget_intersect (DwWidget *widget,
+                            DwRectangle *area,
+                            DwRectangle *intersection)
 {
 #if 1
    DwRectangle parent_area, child_area;
@@ -860,7 +887,7 @@ gint Dw_widget_intersect (DwWidget *widget,
    child_area.width = widget->allocation.width;
    child_area.height = widget->allocation.ascent + widget->allocation.descent;
 
-   if (Dw_rectangle_intersect (&parent_area, &child_area, intersection)) {
+   if (p_Dw_rectangle_intersect (&parent_area, &child_area, intersection)) {
       intersection->x -= widget->allocation.x;
       intersection->y -= widget->allocation.y;
       return TRUE;
@@ -878,8 +905,8 @@ gint Dw_widget_intersect (DwWidget *widget,
 }
 
 
-void Dw_widget_set_parent (DwWidget *widget,
-                           DwWidget *parent)
+void p_Dw_widget_set_parent (DwWidget *widget,
+                             DwWidget *parent)
 {
    gtk_object_ref(GTK_OBJECT (widget));
    gtk_object_sink(GTK_OBJECT (widget));
@@ -896,8 +923,8 @@ void Dw_widget_set_parent (DwWidget *widget,
  * Converting between coordinates.
  */
 
-gint32 Dw_widget_x_viewport_to_world (DwWidget *widget,
-                                      gint16 viewport_x)
+gint32 p_Dw_widget_x_viewport_to_world (DwWidget *widget,
+                                        gint16 viewport_x)
 {
    GtkAdjustment *adjustment;
 
@@ -909,8 +936,8 @@ gint32 Dw_widget_x_viewport_to_world (DwWidget *widget,
 }
 
 
-gint32  Dw_widget_y_viewport_to_world (DwWidget *widget,
-                                       gint16 viewport_y)
+gint32 p_Dw_widget_y_viewport_to_world (DwWidget *widget,
+                                        gint16 viewport_y)
 {
    GtkAdjustment *adjustment;
 
@@ -922,8 +949,8 @@ gint32  Dw_widget_y_viewport_to_world (DwWidget *widget,
 }
 
 
-gint16  Dw_widget_x_world_to_viewport (DwWidget *widget,
-                                       gint32 world_x)
+gint16 p_Dw_widget_x_world_to_viewport (DwWidget *widget,
+                                        gint32 world_x)
 {
    GtkAdjustment *adjustment;
 
@@ -935,8 +962,8 @@ gint16  Dw_widget_x_world_to_viewport (DwWidget *widget,
 }
 
 
-gint16  Dw_widget_y_world_to_viewport (DwWidget *widget,
-                                       gint32 world_y)
+gint16 p_Dw_widget_y_world_to_viewport (DwWidget *widget,
+                                        gint32 world_y)
 {
    GtkAdjustment *adjustment;
 
@@ -978,7 +1005,8 @@ static gboolean Dw_widget_intersect_viewport (DwWidget *widget,
    viewport_area.width = widget->viewport->allocation.width;
    viewport_area.height = widget->viewport->allocation.height;
 
-   if (Dw_rectangle_intersect (&widget_area, &viewport_area, &intersection)) {
+   if (p_Dw_rectangle_intersect (&widget_area, &viewport_area,
+                                 &intersection)) {
       gdk_intersection->x = intersection.x - layout->xoffset;
       gdk_intersection->y = intersection.y - layout->yoffset;
       gdk_intersection->width = intersection.width;
@@ -992,9 +1020,9 @@ static gboolean Dw_widget_intersect_viewport (DwWidget *widget,
 /*
  * ...
  */
-void Dw_widget_queue_draw (DwWidget *widget)
+void p_Dw_widget_queue_draw (DwWidget *widget)
 {
-   Dw_widget_queue_draw_area (
+   p_Dw_widget_queue_draw_area (
       widget,
       0, 0,
       widget->allocation.width,
@@ -1005,11 +1033,11 @@ void Dw_widget_queue_draw (DwWidget *widget)
 /*
  * ...
  */
-void Dw_widget_queue_draw_area (DwWidget *widget,
-                                gint32 x,
-                                gint32 y,
-                                gint32 width,
-                                gint32 height)
+void p_Dw_widget_queue_draw_area (DwWidget *widget,
+                                  gint32 x,
+                                  gint32 y,
+                                  gint32 width,
+                                  gint32 height)
 {
    GdkRectangle gdk_area;
 
@@ -1022,9 +1050,9 @@ void Dw_widget_queue_draw_area (DwWidget *widget,
 /*
  * ...
  */
-void Dw_widget_queue_clear (DwWidget *widget)
+void p_Dw_widget_queue_clear (DwWidget *widget)
 {
-   Dw_widget_queue_clear_area (
+   p_Dw_widget_queue_clear_area (
       widget,
       0, 0,
       widget->allocation.width,
@@ -1035,11 +1063,11 @@ void Dw_widget_queue_clear (DwWidget *widget)
 /*
  * ...
  */
-void Dw_widget_queue_clear_area (DwWidget *widget,
-                                 gint32 x,
-                                 gint32 y,
-                                 gint32 width,
-                                 gint32 height)
+void p_Dw_widget_queue_clear_area (DwWidget *widget,
+                                   gint32 x,
+                                   gint32 y,
+                                   gint32 width,
+                                   gint32 height)
 {
    GdkRectangle gdk_area;
 
@@ -1071,9 +1099,9 @@ static int Dw_widget_queue_resize_idle (gpointer data)
 /*
  * This function should be called, if the widget changed its size.
  */
-void Dw_widget_queue_resize (DwWidget *widget,
-                             gint ref,
-                             gboolean extremes_changed)
+void p_Dw_widget_queue_resize (DwWidget *widget,
+                               gint ref,
+                               gboolean extremes_changed)
 {
    DwWidget *widget2, *child;
    GtkDwViewport *viewport;
@@ -1119,7 +1147,7 @@ void Dw_widget_queue_resize (DwWidget *widget,
  * Add or change an anchor.
  * The widget is responsible for storing a copy of name.
  */
-void Dw_widget_set_anchor (DwWidget *widget, gchar *name, int pos)
+void p_Dw_widget_set_anchor (DwWidget *widget, gchar *name, int pos)
 {
    if (widget->anchors_table == NULL)
       widget->anchors_table = g_hash_table_new(g_str_hash, g_str_equal);
@@ -1134,36 +1162,36 @@ void Dw_widget_set_anchor (DwWidget *widget, gchar *name, int pos)
  * Draw borders and background of a widget part, which allocation is
  * given by (x, y, width, height) (widget coordinates).
  */
-void Dw_widget_draw_box (DwWidget *widget,
-                         DwStyle *style,
-                         DwRectangle *area,
-                         gint32 x,
-                         gint32 y,
-                         gint32 width,
-                         gint32 height)
+void p_Dw_widget_draw_box (DwWidget *widget,
+                           DwStyle *style,
+                           DwRectangle *area,
+                           gint32 x,
+                           gint32 y,
+                           gint32 width,
+                           gint32 height)
 {
    GdkRectangle gdk_area;
    gint32 vx, vy;
 
    if (Dw_widget_intersect_viewport (widget, area->x, area->y,
                                      area->width, area->height, &gdk_area)) {
-      vx = Dw_widget_x_viewport_to_world (widget, 0);
-      vy = Dw_widget_y_viewport_to_world (widget, 0);
+      vx = p_Dw_widget_x_viewport_to_world (widget, 0);
+      vy = p_Dw_widget_y_viewport_to_world (widget, 0);
 
-      Dw_style_draw_border (DW_WIDGET_WINDOW (widget), &gdk_area,
-                            vx, vy,
-                            widget->allocation.x + x,
-                            widget->allocation.y + y,
-                            width, height,
-                            style);
+      p_Dw_style_draw_border (DW_WIDGET_WINDOW (widget), &gdk_area,
+                              vx, vy,
+                              widget->allocation.x + x,
+                              widget->allocation.y + y,
+                              width, height,
+                              style);
 
       if (style->background_color)
-         Dw_style_draw_background (DW_WIDGET_WINDOW (widget), &gdk_area,
-                                   vx, vy,
-                                   widget->allocation.x + x,
-                                   widget->allocation.y + y,
-                                   width, height,
-                                   style);
+         p_Dw_style_draw_background (DW_WIDGET_WINDOW (widget), &gdk_area,
+                                     vx, vy,
+                                     widget->allocation.x + x,
+                                     widget->allocation.y + y,
+                                     width, height,
+                                     style);
    }
 }
 
@@ -1171,25 +1199,25 @@ void Dw_widget_draw_box (DwWidget *widget,
 /*
  * Draw borders and background of a widget.
  */
-void Dw_widget_draw_widget_box (DwWidget *widget,
-                                DwRectangle *area)
+void p_Dw_widget_draw_widget_box (DwWidget *widget,
+                                  DwRectangle *area)
 {
    GdkRectangle gdk_area;
    gint32 vx, vy;
 
    if (Dw_widget_intersect_viewport (widget, area->x, area->y,
                                      area->width, area->height, &gdk_area)) {
-      vx = Dw_widget_x_viewport_to_world (widget, 0);
-      vy = Dw_widget_y_viewport_to_world (widget, 0);
+      vx = p_Dw_widget_x_viewport_to_world (widget, 0);
+      vy = p_Dw_widget_y_viewport_to_world (widget, 0);
 
-      Dw_style_draw_border (DW_WIDGET_WINDOW (widget), &gdk_area,
-                            vx, vy,
-                            widget->allocation.x,
-                            widget->allocation.y,
-                            widget->allocation.width,
-                            widget->allocation.ascent
-                            + widget->allocation.descent,
-                            widget->style);
+      p_Dw_style_draw_border (DW_WIDGET_WINDOW (widget), &gdk_area,
+                              vx, vy,
+                              widget->allocation.x,
+                              widget->allocation.y,
+                              widget->allocation.width,
+                              widget->allocation.ascent
+                              + widget->allocation.descent,
+                              widget->style);
 
       /* - Toplevel widget background colors are set as viewport
        *   background color. This is not crucial for the rendering, but
@@ -1201,13 +1229,13 @@ void Dw_widget_draw_widget_box (DwWidget *widget,
        *   widget->style->background_color is NULL (shining through).
        */
       if (widget->parent && widget->style->background_color)
-         Dw_style_draw_background (DW_WIDGET_WINDOW (widget), &gdk_area,
-                                   vx, vy,
-                                   widget->allocation.x,
-                                   widget->allocation.y,
-                                   widget->allocation.width,
-                                   widget->allocation.ascent
-                                   + widget->allocation.descent,
-                                   widget->style);
+         p_Dw_style_draw_background (DW_WIDGET_WINDOW (widget), &gdk_area,
+                                     vx, vy,
+                                     widget->allocation.x,
+                                     widget->allocation.y,
+                                     widget->allocation.width,
+                                     widget->allocation.ascent
+                                     + widget->allocation.descent,
+                                     widget->style);
    }
 }
